@@ -1,8 +1,7 @@
-from __future__ import absolute_import
-
 from distutils import version
 import os
 import warnings
+import operator
 
 try:
     import matplotlib
@@ -58,34 +57,43 @@ if montage_installed:
         warnings.warn("Python-montage installation is not recent enough (version 0.9.2 or later is required). Disabling Montage-related functionality.")
         montage_installed = False
 
-import aplpy.contour_util as contour_util
-import aplpy.convolve_util as convolve_util
-import aplpy.image_util as image_util
-import aplpy.header as header
-import aplpy.wcs_util as wcs_util
-import aplpy.slicer as slicer
+from . import contour_util
+from . import convolve_util
+from . import image_util
+from . import header as header_util
+from . import wcs_util
+from . import slicer
 
-from aplpy.layers import Layers
-from aplpy.grid import Grid
-from aplpy.ticks import Ticks
-from aplpy.labels import TickLabels
-from aplpy.axis_labels import AxisLabels
-from aplpy.overlays import Beam, ScaleBar
-from aplpy.regions import Regions
-from aplpy.colorbar import Colorbar
-from aplpy.normalize import APLpyNormalize
-from aplpy.frame import Frame
-from aplpy.logger import logger
+from .layers import Layers
+from .grid import Grid
+from .ticks import Ticks
+from .labels import TickLabels
+from .axis_labels import AxisLabels
+from .overlays import Beam, ScaleBar
+from .regions import Regions
+from .colorbar import Colorbar
+from .normalize import APLpyNormalize
+from .frame import Frame
+from .logger import logger
 
-from aplpy.decorators import auto_refresh, fixdocstring
+from .decorators import auto_refresh, fixdocstring
 
-from aplpy.deprecated import Deprecated
+from .deprecated import Deprecated
+
+
+class Parameters():
+    '''
+    A class to contain the current plotting parameters
+    '''
+    pass
 
 
 class FITSFigure(Layers, Regions, Deprecated):
 
     "A class for plotting FITS files."
 
+    _parameters = Parameters()
+
     @auto_refresh
     def __init__(self, data, hdu=0, figure=None, subplot=None,
                  downsample=False, north=False, convention=None,
@@ -96,7 +104,7 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         Required arguments:
 
-            *data*: [ string | pyfits.PrimaryHDU | pyfits.ImageHDU | pywcs.WCS ]
+            *data*: [ string | pyfits.PrimaryHDU | pyfits.ImageHDU | pywcs.WCS | np.ndarray ]
                 Either the filename of the FITS file to open, a pyfits
                 PrimaryHDU or ImageHDU object, or a pywcs WCS object. It is
                 also possible to specify the filename of an RGB image tagged
@@ -160,6 +168,9 @@ class FITSFigure(Layers, Regions, Deprecated):
         #matplotlib.figure.Figure>`_
         '''
 
+        # Set whether to automatically refresh the display
+        self.set_auto_refresh(auto_refresh)
+
         if not 'figsize' in kwargs:
             kwargs['figsize'] = (10, 9)
 
@@ -190,12 +201,15 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         if isinstance(data, pywcs.WCS):
             wcs = data
+            if wcs.naxis != 2:
+                raise ValueError("FITSFigure initialization via WCS objects can only be done with 2-dimensional WCS objects")
             header = wcs.to_header()
-            for i in range(wcs.naxis):
-                header.update('NAXIS%i' % (i + 1), wcs.__getattribute__('naxis%i' % (i + 1)))
+            header.update('NAXIS1', wcs.naxis1)
+            header.update('NAXIS2', wcs.naxis2)
             nx = header['NAXIS%i' % (dimensions[0] + 1)]
             ny = header['NAXIS%i' % (dimensions[1] + 1)]
-            self._hdu = pyfits.ImageHDU(data=np.zeros((ny, nx), dtype=float), header=header)
+            self._data = np.zeros((ny, nx), dtype=float)
+            self._header = header
             self._wcs = wcs_util.WCS(header, dimensions=dimensions, slices=slices)
             self._wcs.nx = nx
             self._wcs.ny = ny
@@ -206,17 +220,17 @@ class FITSFigure(Layers, Regions, Deprecated):
                 logger.warn("north argument is ignored if data passed is a WCS object")
                 north = False
         else:
-            self._hdu, self._wcs = self._get_hdu(data, hdu, north, \
+            self._data, self._header, self._wcs = self._get_hdu(data, hdu, north, \
                 convention=convention, dimensions=dimensions, slices=slices)
-            self._wcs.nx = self._hdu.header['NAXIS%i' % (dimensions[0] + 1)]
-            self._wcs.ny = self._hdu.header['NAXIS%i' % (dimensions[1] + 1)]
+            self._wcs.nx = self._header['NAXIS%i' % (dimensions[0] + 1)]
+            self._wcs.ny = self._header['NAXIS%i' % (dimensions[1] + 1)]
 
         # Downsample if requested
         if downsample:
             nx_new = self._wcs.nx - np.mod(self._wcs.nx, downsample)
             ny_new = self._wcs.ny - np.mod(self._wcs.ny, downsample)
-            self._hdu.data = self._hdu.data[0:ny_new, 0:nx_new]
-            self._hdu.data = image_util.resample(self._hdu.data, downsample)
+            self._data = self._data[0:ny_new, 0:nx_new]
+            self._data = image_util.resample(self._data, downsample)
             self._wcs.nx, self._wcs.ny = nx_new, ny_new
 
         # Open the figure
@@ -253,8 +267,6 @@ class FITSFigure(Layers, Regions, Deprecated):
         self._ax1._wcs = self._wcs
         self._ax2._wcs = self._wcs
 
-        self.set_auto_refresh(auto_refresh)
-
         # Set view to whole FITS file
         self._initialize_view()
 
@@ -273,7 +285,7 @@ class FITSFigure(Layers, Regions, Deprecated):
         self._initialize_layers()
 
         # Find generating function for vmin/vmax
-        self._auto_v = image_util.percentile_function(self._hdu.data)
+        self._auto_v = image_util.percentile_function(self._data)
 
         # Set image holder to be empty
         self.image = None
@@ -315,6 +327,10 @@ class FITSFigure(Layers, Regions, Deprecated):
             else:
                 hdu = hdulist[hdu]
 
+        elif type(data) == np.ndarray:
+
+            hdu = pyfits.ImageHDU(data)
+
         elif isinstance(data, pyfits.PrimaryHDU) or isinstance(data, pyfits.ImageHDU):
 
             hdu = data
@@ -325,10 +341,17 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         else:
 
-            raise Exception("data argument should either be a filename, or an HDU instance from pyfits.")
+            raise Exception("data argument should either be a filename, an HDU instance from pyfits, a pywcs.WCS object, or a Numpy array.")
 
-        # Extract slices
-        hdu = slicer.slice_hypercube(hdu, dimensions=dimensions, slices=slices)
+        # Check dimensions= argument
+        if type(dimensions) not in [list, tuple]:
+            raise ValueError('dimensions= should be a list or a tuple')
+        if len(set(dimensions)) != 2 or len(dimensions) != 2:
+            raise ValueError("dimensions= should be a tuple of two different values")
+        if dimensions[0] < 0 or dimensions[0] > hdu.header['NAXIS'] - 1:
+            raise ValueError('values of dimensions= should be between %i and %i' % (0, hdu.header['NAXIS'] - 1))
+        if dimensions[1] < 0 or dimensions[1] > hdu.header['NAXIS'] - 1:
+            raise ValueError('values of dimensions= should be between %i and %i' % (0, hdu.header['NAXIS'] - 1))
 
         # Reproject to face north if requested
         if north:
@@ -336,17 +359,37 @@ class FITSFigure(Layers, Regions, Deprecated):
                 raise Exception("Both the Montage command-line tools and the Python-montage module are required to use the north= argument")
             hdu = montage.reproject_hdu(hdu, north_aligned=True)
 
+        # Now copy the data and header to new objects, since in PyFITS the two
+        # attributes are linked, which can lead to confusing behavior. We just
+        # need to copy the header to avoid memory issues - as long as one item
+        # is copied, the two variables are decoupled.
+        data = hdu.data
+        header = hdu.header.copy()
+        del hdu
+
+        # If slices wasn't specified, check if we can guess
+        shape = data.shape
+        if len(shape) > 2:
+            n_total = reduce(operator.mul, shape)
+            n_image = shape[len(shape) - 1 - dimensions[0]] \
+                    * shape[len(shape) - 1 - dimensions[1]]
+            if n_total == n_image:
+                slices = [0 for i in range(1, len(shape) - 1)]
+                logger.info("Setting slices=%s" % str(slices))
+
+        # Extract slices
+        data = slicer.slice_hypercube(data, header, dimensions=dimensions, slices=slices)
+
         # Check header
-        hdu.header = header.check(hdu.header, convention=convention, dimensions=dimensions)
+        header = header_util.check(header, convention=convention, dimensions=dimensions)
 
         # Parse WCS info
-        wcs = wcs_util.WCS(hdu.header, dimensions=dimensions, slices=slices)
         try:
-            wcs = wcs_util.WCS(hdu.header, dimensions=dimensions, slices=slices)
+            wcs = wcs_util.WCS(header, dimensions=dimensions, slices=slices)
         except:
             raise Exception("An error occured while parsing the WCS information")
 
-        return hdu, wcs
+        return data, header, wcs
 
     @auto_refresh
     def set_xaxis_coord_type(self, coord_type):
@@ -395,38 +438,35 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         Required Arguments:
 
-            *x*: [ float ]
-                Longitude of the position to center on (degrees)
-
-            *y*: [ float ]
-                Latitude of the position to center on (degrees)
+            *x, y*: [ float ]
+                Coordinates to center on
 
         Optional Keyword Arguments:
 
             Either the radius or width/heigh arguments should be specified.
 
             *radius*: [ float ]
-                Radius of the region to view (degrees). This produces a square plot.
+                Radius of the region to view. This produces a square plot.
 
             *width*: [ float ]
-                Width of the region to view (degrees). This should be given in
+                Width of the region to view. This should be given in
                 conjunction with the height argument.
 
             *height*: [ float ]
-                Height of the region to view (degrees). This should be given in
+                Height of the region to view. This should be given in
                 conjunction with the width argument.
         '''
 
         xpix, ypix = wcs_util.world2pix(self._wcs, x, y)
 
-        degperpix = wcs_util.degperpix(self._wcs)
+        pixel_scale = wcs_util.pixel_scale(self._wcs)
 
         if radius:
-            dx_pix = radius / degperpix
-            dy_pix = radius / degperpix
+            dx_pix = radius / pixel_scale
+            dy_pix = radius / pixel_scale
         elif width and height:
-            dx_pix = width / degperpix / 2.
-            dy_pix = height / degperpix / 2.
+            dx_pix = width / pixel_scale / 2.
+            dy_pix = height / pixel_scale / 2.
         else:
             raise Exception("Need to specify either radius= or width= and height= arguments")
 
@@ -636,11 +676,11 @@ class FITSFigure(Layers, Regions, Deprecated):
             self.image.set_cmap(cmap=cmap)
             self.image.origin = 'lower'
             self.image.set_interpolation(interpolation)
-            self.image.set_data(convolve_util.convolve(self._hdu.data,
+            self.image.set_data(convolve_util.convolve(self._data,
                                                        smooth=smooth,
                                                        kernel=kernel))
         else:
-            self.image = self._ax1.imshow(convolve_util.convolve(self._hdu.data, smooth=smooth, kernel=kernel), cmap=cmap, interpolation=interpolation, origin='lower', extent=self._extent, norm=normalizer, aspect=aspect)
+            self.image = self._ax1.imshow(convolve_util.convolve(self._data, smooth=smooth, kernel=kernel), cmap=cmap, interpolation=interpolation, origin='lower', extent=self._extent, norm=normalizer, aspect=aspect)
 
         xmin, xmax = self._ax1.get_xbound()
         if xmin == 0.0:
@@ -804,12 +844,12 @@ class FITSFigure(Layers, Regions, Deprecated):
         elif not colors:
             cmap = mpl.cm.get_cmap('jet')
 
-        hdu_contour, wcs_contour = self._get_hdu(data, hdu, False, \
+        data_contour, header_contour, wcs_contour = self._get_hdu(data, hdu, False, \
             convention=convention, dimensions=dimensions, slices=slices)
-        wcs_contour.nx = hdu_contour.header['NAXIS%i' % (dimensions[0] + 1)]
-        wcs_contour.ny = hdu_contour.header['NAXIS%i' % (dimensions[1] + 1)]
+        wcs_contour.nx = header_contour['NAXIS%i' % (dimensions[0] + 1)]
+        wcs_contour.ny = header_contour['NAXIS%i' % (dimensions[1] + 1)]
 
-        image_contour = convolve_util.convolve(hdu_contour.data, smooth=smooth, kernel=kernel)
+        image_contour = convolve_util.convolve(data_contour, smooth=smooth, kernel=kernel)
         extent_contour = (0.5, wcs_contour.nx + 0.5, 0.5, wcs_contour.ny + 0.5)
 
         if type(levels) == int:
@@ -904,7 +944,7 @@ class FITSFigure(Layers, Regions, Deprecated):
                 The y positions of the circles (in world coordinates)
 
             *radius*: [integer | float | list | numpy.ndarray ]
-                The radii of the circles in degrees
+                The radii of the circles (in world coordinates)
 
         Optional Keyword Arguments:
 
@@ -976,10 +1016,10 @@ class FITSFigure(Layers, Regions, Deprecated):
                The y positions of the ellipses (in world coordinates)
 
            *width*: [integer | float | list | numpy.ndarray ]
-               The width of the ellipse in degrees
+               The width of the ellipse (in world coordinates)
 
            *height*: [integer | float | list | numpy.ndarray ]
-               The height of the ellipse in degrees
+               The height of the ellipse (in world coordinates)
 
        Optional Keyword Arguments:
 
@@ -1067,10 +1107,10 @@ class FITSFigure(Layers, Regions, Deprecated):
                The y positions of the rectangles (in world coordinates)
 
            *width*: [integer | float | list | numpy.ndarray ]
-               The width of the rectangle in degrees
+               The width of the rectangle (in world coordinates)
 
            *height*: [integer | float | list | numpy.ndarray ]
-               The height of the rectangle in degrees
+               The height of the rectangle (in world coordinates)
 
        Optional Keyword Arguments:
 
@@ -1424,7 +1464,7 @@ class FITSFigure(Layers, Regions, Deprecated):
                 the display can be refreshed manually using the refresh()
                 method
         '''
-        self._figure._auto_refresh = refresh
+        self._parameters.auto_refresh = refresh
 
     def refresh(self, force=True):
         '''
@@ -1438,7 +1478,7 @@ class FITSFigure(Layers, Regions, Deprecated):
                 refreshed whatever the auto refresh setting is set to.
                 The default is True.
         '''
-        if self._figure._auto_refresh or force:
+        if self._parameters.auto_refresh or force:
             self._figure.canvas.draw()
 
     def save(self, filename, dpi=None, transparent=False, adjust_bbox=True, max_dpi=300):
@@ -1667,7 +1707,7 @@ class FITSFigure(Layers, Regions, Deprecated):
             del self.beam
 
     @auto_refresh
-    def add_scalebar(self, *args, **kwargs):
+    def add_scalebar(self, length, *args, **kwargs):
         '''
         Add a scalebar to the current figure
 
@@ -1684,7 +1724,7 @@ class FITSFigure(Layers, Regions, Deprecated):
             raise Exception("Scalebar already exists")
         try:
             self.scalebar = ScaleBar(self)
-            self.scalebar.show(*args, **kwargs)
+            self.scalebar.show(length, *args, **kwargs)
         except:
             del self.scalebar
             raise
