@@ -1,12 +1,10 @@
-from __future__ import absolute_import
-
 import warnings
 
 import matplotlib.axes as maxes
 from mpl_toolkits.axes_grid import make_axes_locatable
 from matplotlib.font_manager import FontProperties
 
-from aplpy.decorators import auto_refresh, fixdocstring
+from .decorators import auto_refresh, fixdocstring
 
 # As of matplotlib 0.99.1.1, any time a colorbar property is updated, the axes
 # need to be removed and re-created. This has been fixed in svn r8213 but we
@@ -20,6 +18,10 @@ class Colorbar(object):
         self._figure = parent._figure
         self._colorbar_axes = None
         self._parent = parent
+
+        # Save plotting parameters (required for @auto_refresh)
+        self._parameters = parent._parameters
+
         self._base_settings = {}
         self._label_fontproperties = FontProperties()
 
