@@ -1,9 +1,7 @@
-from __future__ import absolute_import
-
 import numpy as np
 from matplotlib.path import Path
 
-import aplpy.wcs_util as wcs_util
+from . import wcs_util
 
 
 def transform(contours, wcs_in, wcs_out, filled=False, overlap=False):
