@@ -1,8 +1,6 @@
-from __future__ import absolute_import
-
 import threading
 
-from aplpy.decorator import decorator
+from .decorator import decorator
 
 
 mydata = threading.local()
@@ -26,7 +24,7 @@ def _auto_refresh(f, *args, **kwargs):
     finally:
         mydata.nesting -= 1
         if hasattr(args[0], '_figure'):
-            if refresh and mydata.nesting == 0 and args[0]._figure._auto_refresh:
+            if refresh and mydata.nesting == 0 and args[0]._parameters.auto_refresh:
                 args[0]._figure.canvas.draw()
 
 
