@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import warnings
 
 from mpl_toolkits.axes_grid.anchored_artists \
@@ -9,8 +7,8 @@ import numpy as np
 from matplotlib.patches import FancyArrowPatch
 from matplotlib.font_manager import FontProperties
 
-import aplpy.wcs_util as wcs_util
-from aplpy.decorators import auto_refresh
+from . import wcs_util
+from .decorators import auto_refresh
 
 corners = {}
 corners['top right'] = 1
@@ -44,7 +42,7 @@ class Compass(object):
         Required Arguments:
 
             *length*:[ float ]
-                The length of the scalebar, in degrees
+                The length of the scalebar
 
         Optional Keyword Arguments:
 
@@ -113,9 +111,9 @@ class Compass(object):
 
         len_pix = length * (ymax - ymin)
 
-        degperpix = wcs_util.degperpix(self._wcs)
+        pixel_scale = wcs_util.pixel_scale(self._wcs)
 
-        len_deg = len_pix * degperpix
+        len_deg = len_pix * pixel_scale
 
         # Should really only do tiny displacement then magnify the vectors - important if there is curvature
 
@@ -148,6 +146,9 @@ class ScaleBar(object):
         self._wcs = parent._wcs
         self._figure = parent._figure
 
+        # Save plotting parameters (required for @auto_refresh)
+        self._parameters = parent._parameters
+
         # Initialize settings
         self._base_settings = {}
         self._scalebar_settings = {}
@@ -164,7 +165,7 @@ class ScaleBar(object):
         Required Arguments:
 
             *length*:[ float ]
-                The length of the scalebar, in degrees
+                The length of the scalebar
 
         Optional Keyword Arguments:
 
@@ -193,9 +194,9 @@ class ScaleBar(object):
         self._base_settings['borderpad'] = borderpad
         self._base_settings['pad'] = pad
 
-        degperpix = wcs_util.degperpix(self._wcs)
+        pixel_scale = wcs_util.pixel_scale(self._wcs)
 
-        length = length / degperpix
+        length = length / pixel_scale
 
         try:
             self._scalebar.remove()
@@ -229,7 +230,7 @@ class ScaleBar(object):
     @auto_refresh
     def set_length(self, length):
         '''
-        Set the length of the scale bar, in degrees.
+        Set the length of the scale bar.
         '''
         self.show(length, **self._base_settings)
         self._set_scalebar_properties(**self._scalebar_settings)
@@ -406,7 +407,7 @@ class Beam(object):
 
         # Retrieve info from parent figure
         self._figure = parent._figure
-        self._hdu = parent._hdu
+        self._header = parent._header
         self._ax = parent._ax1
         self._wcs = parent._wcs
 
@@ -455,15 +456,15 @@ class Beam(object):
         '''
 
         if isinstance(major, basestring):
-            major = self._hdu.header[major]
+            major = self._header[major]
 
         if isinstance(minor, basestring):
-            minor = self._hdu.header[minor]
+            minor = self._header[minor]
 
         if isinstance(angle, basestring):
-            angle = self._hdu.header[angle]
+            angle = self._header[angle]
 
-        degperpix = wcs_util.degperpix(self._wcs)
+        pixel_scale = wcs_util.pixel_scale(self._wcs)
 
         self._base_settings['minor'] = minor
         self._base_settings['major'] = major
@@ -473,8 +474,8 @@ class Beam(object):
         self._base_settings['borderpad'] = borderpad
         self._base_settings['pad'] = pad
 
-        minor /= degperpix
-        major /= degperpix
+        minor /= pixel_scale
+        major /= pixel_scale
 
         try:
             self._beam.remove()
