@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 try:
     import pyregion
     pyregion_installed = True
@@ -11,7 +9,7 @@ def _check_pyregion_installed():
     if not pyregion_installed:
         raise Exception("The pyregion package is required to load region files")
 
-from aplpy.decorators import auto_refresh
+from .decorators import auto_refresh
 
 
 class Regions:
@@ -70,7 +68,7 @@ class Regions:
 
         _check_pyregion_installed()
 
-        PC, TC = ds9(region_file, self._hdu.header, **kwargs)
+        PC, TC = ds9(region_file, self._header, **kwargs)
 
         #ffpc = self._ax1.add_collection(PC)
         PC.add_to_axes(self._ax1)
