@@ -1,15 +1,13 @@
-from __future__ import absolute_import
-
 import warnings
 
 import numpy as np
 from matplotlib.pyplot import Locator
 
-import aplpy.wcs_util as wcs_util
-import aplpy.angle_util as au
-import aplpy.scalar_util as su
-import aplpy.math_util as math_util
-from aplpy.decorators import auto_refresh
+from . import wcs_util
+from . import angle_util as au
+from . import scalar_util as su
+from . import math_util
+from .decorators import auto_refresh
 
 
 class Ticks(object):
@@ -24,6 +22,9 @@ class Ticks(object):
         self._figure = parent._figure
         self._parent = parent
 
+        # Save plotting parameters (required for @auto_refresh)
+        self._parameters = parent._parameters
+
         # Set tick positions
         self._ax1.yaxis.tick_left()
         self._ax1.xaxis.tick_bottom()
@@ -79,7 +80,7 @@ class Ticks(object):
                 self._ax2.xaxis.apl_tick_spacing = au.Angle(degrees=spacing, latitude=self._wcs.xaxis_coord_type == 'latitude')
             else:
                 try:
-                    su._check_format_spacing_consistency(self._ax1.xaxis.apl_label_form, au.Angle(degrees=spacing, latitude=self._wcs.xaxis_coord_type == 'latitude'))
+                    su._check_format_spacing_consistency(self._ax1.xaxis.apl_label_form, spacing)
                 except au.InconsistentSpacing:
                     warnings.warn("WARNING: Requested tick spacing format cannot be shown by current label format. The tick spacing will not be changed.")
                     return
@@ -114,7 +115,7 @@ class Ticks(object):
                 self._ax2.yaxis.apl_tick_spacing = au.Angle(degrees=spacing, latitude=self._wcs.yaxis_coord_type == 'latitude')
             else:
                 try:
-                    su._check_format_spacing_consistency(self._ax1.yaxis.apl_label_form, au.Angle(degrees=spacing, latitude=self._wcs.yaxis_coord_type == 'latitude'))
+                    su._check_format_spacing_consistency(self._ax1.yaxis.apl_label_form, spacing)
                 except au.InconsistentSpacing:
                     warnings.warn("WARNING: Requested tick spacing format cannot be shown by current label format. The tick spacing will not be changed.")
                     return
@@ -213,6 +214,78 @@ class Ticks(object):
         self._ax2.xaxis.get_minor_locator().subticks = frequency
         self._ax2.yaxis.get_minor_locator().subticks = frequency
 
+    @auto_refresh
+    def show(self):
+        """
+        Show the x- and y-axis ticks
+        """
+        self.show_x()
+        self.show_y()
+
+    @auto_refresh
+    def hide(self):
+        """
+        Hide the x- and y-axis ticks
+        """
+        self.hide_x()
+        self.hide_y()
+
+    @auto_refresh
+    def show_x(self):
+        """
+        Show the x-axis ticks
+        """
+        for line in self._ax1.xaxis.get_ticklines():
+            line.set_visible(True)
+        for line in self._ax2.xaxis.get_ticklines():
+            line.set_visible(True)
+        for line in self._ax1.xaxis.get_minorticklines():
+            line.set_visible(True)
+        for line in self._ax2.xaxis.get_minorticklines():
+            line.set_visible(True)
+
+    @auto_refresh
+    def hide_x(self):
+        """
+        Hide the x-axis ticks
+        """
+        for line in self._ax1.xaxis.get_ticklines():
+            line.set_visible(False)
+        for line in self._ax2.xaxis.get_ticklines():
+            line.set_visible(False)
+        for line in self._ax1.xaxis.get_minorticklines():
+            line.set_visible(False)
+        for line in self._ax2.xaxis.get_minorticklines():
+            line.set_visible(False)
+
+    @auto_refresh
+    def show_y(self):
+        """
+        Show the y-axis ticks
+        """
+        for line in self._ax1.yaxis.get_ticklines():
+            line.set_visible(True)
+        for line in self._ax2.yaxis.get_ticklines():
+            line.set_visible(True)
+        for line in self._ax1.yaxis.get_minorticklines():
+            line.set_visible(True)
+        for line in self._ax2.yaxis.get_minorticklines():
+            line.set_visible(True)
+
+    @auto_refresh
+    def hide_y(self):
+        """
+        Hide the y-axis ticks
+        """
+        for line in self._ax1.yaxis.get_ticklines():
+            line.set_visible(False)
+        for line in self._ax2.yaxis.get_ticklines():
+            line.set_visible(False)
+        for line in self._ax1.yaxis.get_minorticklines():
+            line.set_visible(False)
+        for line in self._ax2.yaxis.get_minorticklines():
+            line.set_visible(False)
+
 
 class WCSLocator(Locator):
 
